// Persistence of Vision Ray Tracer Scene Description File
// File: povcomp2004.pov
// Vers: 3.6
// Desc: scene file for "Crystal Sentinels"
// Date: January 6th, 2005
// Auth: Philip Chan

// final render
// +W1600 +H1200 +A0.3 -uv
// detail view (rocks, mushrooms, etc.)
// +FP +W4800 +H3600 +A0.3 -uv +sc1540 +ec2339 +sr1790 +er2589
// detail view (planet)
// +FP +W4800 +H3600 +A0.3 -uv +sc3810 +ec4609 +sr45 +er844

#include "colors.inc"
#include "spiral.inc"
#include "fieldcam.inc"
#include "metals.inc"
#include "math.inc"
#include "functions.inc"

#declare hf_smooth = true;
#declare hf_detail = true;
#declare hf_texture_detail = 1;
#declare water_detail = true;
#declare rocks = true;
#declare rock_detail = true;
#declare mushrooms = true;
#declare atmosphere = true;
#declare media_effects = true;
#declare cages = true;
#declare cage_detail = true;
#declare cage_lights = true;
#declare crystals = true;
#declare crystal_detail = true;
#declare crystal_light = true;
#declare use_rad = true;

#declare test_light = false;
#declare lighting_detail = false;
#declare photon = false;

#declare rad_up_factor = 0;  // percentage increase for "higher is better" values
#declare rad_down_factor = 0;  // percentage decrease for "lower is better" values

global_settings {
  //assumed_gamma 1
  //ambient_light colour rgb <1, 0.75, 1>
  adc_bailout 1/128
  max_trace_level 8
  #if ( photon )
    photons {
      spacing 0.02
      radius ,2
      autostop 1
    }
  #end
  #if ( use_rad )
  	//ambient_light 0
  	radiosity {
  		pretrace_start 0.08*( 1 - rad_down_factor )
  		pretrace_end   0.04*( 1 - rad_down_factor )
  		count floor( 35*( 1 + rad_up_factor ) )
  		
  		nearest_count floor( 5*( 1 + rad_up_factor ) )
  		error_bound 1.3
  		recursion_limit 3
  		
  		low_error_factor 0.5*( 1 - rad_down_factor )
  		gray_threshold 0.5
  		minimum_reuse 0.015*( 1 - rad_down_factor )
  		//brightness 1
  		
  		adc_bailout 0.01/2
  	}
  #end
}

#declare CamPos = <0, 50, 0>;
#declare CamLook = <0, 0, 250>;
camera {
	FieldCam (CamPos, CamLook)
	perspective
	location CamPos
	transform { NoFall }
	look_at CamLook
}
/*
camera {
  location <0, 50, 0>
  look_at <0, 0, 250>
}
*/

//*** Begin Shared Parameters ***\\
#declare Spiral_Loc = array[6]
{
  <1, 0, 110>, <-100, 0, 750>, <200, 0, 400>, <75, 0, 300>, <-100, 0, 250>, <-400, 0, 800>
}
//*** End Shared Parameters ***\\

//*** Begin Atmospheric Stuff *** \\
#if ( atmosphere )

// atmospheric media
media {
  scattering {
    1,
    colour rgb 1
    extinction 0.0002
  }
  density {
    agate
    turbulence 0.7
    density_map {
      [ 0.00 colour rgbt <0.0, 0.0, 0.0, 1.0> ]
      [ 0.50 colour rgbt <0.0, 0.0, 0.0, 1.0> ]
      [ 1.00 colour rgbt <0.4, 0.2, 0.4, 0.0> ]
    }
    scale <500, 100, 100>
  }
  collect off
}

// water fog (makes the ground fade away under the water)
fog {
  distance 2
  turbulence 0.1
  colour rgb <0.3, 0.15, 0.3>
  fog_type 2
  fog_offset -20
  fog_alt 1
}

// stars
difference {
  plane { -z, -6000 }
  plane { -z, -6001 }
  cylinder { <2135, 910, 5999>, <2135, 910, 6002>, 810 }
  difference {
    cylinder { <0, 0, 5999>, <0, 0, 6002>, 1685
      scale <1, 0.25, 1>
      rotate z*19
      translate <2285, 855, 0>
    }
    cylinder { <0, 0, 5998>, <0, 0, 6003>, 1100
      scale <1, 0.25, 1>
      rotate z*19
      translate <2190, 875, 0>
    }
  }
  no_shadow
  pigment {
    leopard
    turbulence 0.5
    colour_map {
      [0.00 colour rgbt <0, 0, 0, 1>]
      [0.97 colour rgbt <0, 0, 0, 1>]
      [0.99 colour rgb <2.5, 2.5, 2.5>]
      [1.00 colour rgb <2.5, 2.5, 2.5>]
    }
    scale 10
    translate <41.76, 38.78, 1.438>
    rotate <190.44, 61.2, 331.92>
  }
}    

// clouds (blue layer)
difference {
  plane { -z, -5500 }
  plane { -z, -5501 }
  //plane { y, 0 }
  no_shadow
  texture {
    pigment {
      bozo
      turbulence 0.5
      omega 0.75
      colour_map {
        [ 0.00, colour rgbt <0.00, 0.35, 0.75, 0.00> ]
        [ 0.20, colour rgbt <0.00, 0.10, 0.28, 0.50> ]
        [ 0.35, colour rgbt <0.00, 0.00, 0.00, 1.00> ]
        [ 0.65, colour rgbt <0.00, 0.00, 0.00, 1.00> ]
        [ 0.80, colour rgbt <0.00, 0.10, 0.28, 0.50> ]
        [ 1.00, colour rgbt <0.00, 0.35, 0.75, 0.00> ]
        /*[ 0.00 colour rgbft <0.4, 0.6, 0.8, 0.2, 0.0> ]
        [ 0.25 colour rgbft <0.2, 0.4, 0.4, 0.4, 0.3> ]
        [ 0.30 colour rgbft <0.3, 0.5, 0.6, 0.2, 0.8> ]
        [ 0.35 colour rgbft <0.3, 0.5, 0.6, 0.1, 0.9> ]
        [ 0.40 colour rgbft <0.3, 0.5, 0.6, 0.0, 1.0> ]
        [ 0.60 colour rgbft <0.3, 0.5, 0.6, 0.0, 1.0> ]
        [ 0.65 colour rgbft <0.3, 0.5, 0.6, 0.1, 0.9> ]
        [ 0.70 colour rgbft <0.3, 0.5, 0.6, 0.2, 0.8> ]
        [ 0.75 colour rgbft <0.2, 0.4, 0.4, 0.4, 0.3> ]
        [ 1.00 colour rgbft <0.4, 0.6, 0.8, 0.2, 0.0> ]*/
      }
      scale <1500, 500, 250>
    }
    finish { ambient 1.0 }
  }
}
// more clouds (purple atmosphere layer)
difference {
  plane { -z, -5000 }
  plane { -z, -5001 }
  //plane { y, 0 }
  no_shadow
  texture {
    pigment {
      spotted
      turbulence 0.3
      colour_map {
        [ 0.00 colour rgbft <0.4, 0.2, 0.4, 0.1, 0.9> ]
        [ 0.02 colour rgbft <0.4, 0.2, 0.4, 0.2, 0.8> ]
        [ 0.10 colour rgbft <0.4, 0.2, 0.4, 0.3, 0.7> ]
        [ 0.30 colour rgbft <0.4, 0.2, 0.4, 0.5, 0.5> ]
        [ 0.50 colour rgbft <0.6, 0.4, 0.6, 0.5, 0.5> ]
        [ 0.70 colour rgbft <0.4, 0.2, 0.4, 0.5, 0.5> ]
        [ 0.90 colour rgbft <0.4, 0.2, 0.4, 0.3, 0.7> ]
        [ 0.98 colour rgbft <0.4, 0.2, 0.4, 0.2, 0.8> ]
        [ 1.00 colour rgbft <0.4, 0.2, 0.4, 0.1, 0.9> ]
      }
      scale <2000, 200, 200>
    }
    finish { ambient 1.0 }
  }
}

#end
// *** End Atmospheric Stuff *** \\

// *** Begin Terrain Stuff *** \\
#declare Water = material {
  texture {
    pigment { colour rgbt <0.4, 0.4, 2.0, 0.5> }
    finish {
      conserve_energy
      reflection {
        0.15
        metallic
      }    
    }
    normal {
      bozo 3.0
      turbulence 0.5
      scale 2.5
    }
  }
  interior {
    ior 1.33
  }
}

// terrain
#declare Terrain = height_field {
	tga "terrain_hf.tga"
	#if ( hf_smooth )
		smooth
	#end
	scale <1250, 312.5, 1250>
	translate <-625, -75, 0>
}

#switch ( hf_texture_detail )
  #case ( 0 )
    #declare T_Terrain = texture {
      pigment {
      	granite
        colour_map {
      		[ 0.000 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.090 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.090 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.180 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.180 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.270 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.270 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.360 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.360 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.450 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.450 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.550 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.550 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.640 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.640 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.730 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.730 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.820 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.820 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.910 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.910 colour rgb <1.00, 0.30, 1.00> ]
      		[ 1.000 colour rgb <1.00, 0.30, 1.00> ]
      	}
      	scale 0.5
      }
      finish { ambient 0.2 }
    }
  #break  
  #case ( 1 )
    #declare T_Terrain1 = texture {
      pigment {
        gradient y
        turbulence 0.15
        octaves 4
        colour_map {
          [ 0.00 colour rgb <1.00, 0.00, 1.00> ]
          [ 0.20 colour rgb <1.00, 0.00, 1.00> ]
          [ 0.70 colour rgb <0.70, 0.70, 0.70> ]
          [ 1.00 colour rgb <0.70, 0.70, 0.70> ]
        }
        scale <20, 312.5, 10>
        translate -y*75
      }
      finish { ambient 0.2 }
    }
    #declare T_Terrain2 = texture {
      pigment {
        bozo
        turbulence 0.5
        omega 0.75
        colour_map {
          [0.00 colour rgbf <1.000, 1.000, 1.000, 1.000>]
          [0.40 colour rgbf <0.800, 0.800, 0.800, 0.800>]
          [0.75 colour rgbf <0.632, 0.612, 0.688, 0.698>]
          [1.00 colour rgbf <0.546, 0.597, 0.676, 0.953>]
        }
      }
      scale 0.3
    }
    #declare T_Terrain = texture { T_Terrain1 } texture { T_Terrain2 }
  #break
  #case ( 2 )
    #declare T_Terrain1 = texture {
      pigment {
      	granite
        colour_map {
      		[ 0.000 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.090 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.090 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.180 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.180 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.270 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.270 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.360 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.360 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.450 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.450 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.550 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.550 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.640 colour rgb <1.00, 0.30, 1.00> ]
      		[ 0.640 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.730 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.730 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.820 colour rgb <0.70, 0.00, 0.70> ]
      		[ 0.820 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.910 colour rgb <1.00, 0.00, 1.00> ]
      		[ 0.910 colour rgb <1.00, 0.30, 1.00> ]
      		[ 1.000 colour rgb <1.00, 0.30, 1.00> ]
      	}
      	scale 0.5
      }
      finish { ambient 0.2 }
    }
    #declare T_Terrain = texture { T_Terrain1 }
    /*#declare T_Terrain2 = texture {
      pigment {
      	granite
        colour_map {
      		[ 0.000 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.090 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.090 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.180 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.180 colour rgb <0.85, 0.70, 0.85> ]
      		[ 0.270 colour rgb <0.85, 0.70, 0.85> ]
      		[ 0.270 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.360 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.360 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.450 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.450 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.550 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.550 colour rgb <0.85, 0.70, 0.85> ]
      		[ 0.640 colour rgb <0.85, 0.70, 0.85> ]
      		[ 0.640 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.730 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.730 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.820 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.820 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.910 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.910 colour rgb <0.85, 0.70, 0.85> ]
      		[ 1.000 colour rgb <0.85, 0.70, 0.85> ]
      	}
      	scale 0.5
      }
      finish { ambient 0.2 }
    }*/
    #declare T_Terrain2 = texture {
      pigment {
      	granite
        colour_map {
      		[ 0.000 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.141 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.141 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.281 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.281 colour rgb <0.85, 0.70, 0.85> ]
      		[ 0.422 colour rgb <0.85, 0.70, 0.85> ]
      		[ 0.422 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.563 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.563 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.703 colour rgb <0.49, 0.49, 0.49> ]
      		[ 0.703 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.859 colour rgb <0.70, 0.70, 0.70> ]
      		[ 0.859 colour rgb <0.85, 0.70, 0.85> ]
      		[ 1.000 colour rgb <0.85, 0.70, 0.85> ]
      	}
      	scale 0.5
      }
      finish { ambient 0.2 }
    }
    #declare T_Terrain = texture {
      slope {
        -y, 0, 0.5
        altitude 2*y, -75, 237.5
      }
      turbulence 0.1
      texture_map {
        [ 0.00 T_Terrain1 ]
        [ 0.20 T_Terrain1 ]
        [ 1.00 T_Terrain2 ]
      }
    }
  #break
#end  
// *** End Terrain Stuff *** \\

// *** Rocks Stuff *** \\
// Pigment noise function for rocks and walls
#declare fn_Pigm = function {
	pigment {
		agate
		color_map {
			[0 color rgb 0]
			[1 color rgb 1]
		}
	}
}

#declare T_Rock = texture {
   pigment {
      agate
      turbulence 2.0
      colour_map {
         [0.00 colour rgb <0.25, 0.25, 0.25> ]
         [0.10 colour rgb <0.25, 0.25, 0.25> ]
         [0.20 colour rgb <0.40, 0.40, 0.40> ]
         [0.60 colour rgb <0.40, 0.40, 0.40> ]
         [0.70 colour rgb <0.55, 0.55, 0.55> ]
         [0.75 colour rgb <0.55, 0.55, 0.55> ]
         [0.80 colour rgb <0.40, 0.40, 0.40> ]
         [1.00 colour rgb <0.40, 0.40, 0.40> ]
      }
   }
   finish { 
      reflection 0.0
      diffuse 1.0
   }
}
/*texture {
  pigment {
  	granite
    colour_map {
  		[ 0.000 colour rgb <0.49, 0.49, 0.49> ]
  		[ 0.141 colour rgb <0.49, 0.49, 0.49> ]
  		[ 0.141 colour rgb <0.70, 0.70, 0.70> ]
  		[ 0.281 colour rgb <0.70, 0.70, 0.70> ]
  		[ 0.281 colour rgb <0.85, 0.70, 0.85> ]
  		[ 0.422 colour rgb <0.85, 0.70, 0.85> ]
  		[ 0.422 colour rgb <0.70, 0.70, 0.70> ]
  		[ 0.563 colour rgb <0.70, 0.70, 0.70> ]
  		[ 0.563 colour rgb <0.49, 0.49, 0.49> ]
  		[ 0.703 colour rgb <0.49, 0.49, 0.49> ]
  		[ 0.703 colour rgb <0.70, 0.70, 0.70> ]
  		[ 0.859 colour rgb <0.70, 0.70, 0.70> ]
  		[ 0.859 colour rgb <0.85, 0.70, 0.85> ]
  		[ 1.000 colour rgb <0.85, 0.70, 0.85> ]
  	}
  	scale 0.75
  }
  //finish { ambient 0.2 }
}*/
// *** End Rocks Stuff *** \\

// *** Begin Planet Stuff *** \\

#declare Bands = pigment {
  gradient y
  turbulence 0.35
  colour_map {
    [ 0.00 colour rgb <0.00, 1.25, 1.25> ]
    [ 0.20 colour rgb <0.25, 1.00, 1.00> ]
    [ 0.30 colour rgb <0.25, 1.00, 1.00> ]
    [ 0.35 colour rgb <0.00, 1.25, 1.25> ]
    [ 0.85 colour rgb <0.00, 1.00, 1.00> ]
    [ 1.00 colour rgb <0.00, 1.00, 1.00> ]
  }
}

#declare Swirls = pigment {
  bozo
  turbulence 0.4
  colour_map {
    [ 0.00 colour rgb <0.00, 1.00, 1.00> ]
    [ 0.20 colour rgb <0.25, 1.00, 1.00> ]
    [ 0.40 colour rgb <0.25, 1.00, 1.00> ]
    [ 0.60 colour rgb <0.00, 1.00, 1.00> ]
    [ 1.00 colour rgb <0.00, 1.25, 1.25> ]
  }
}

#declare T_Planet = texture {
  pigment {
    colour rgb <0, 1, 1>
  }
  finish {
    ambient 0//.05
    diffuse 1
    
  }
}

// planet
light_group {
  light_source {
    //<-1500, 5000, 12500>
    <-4000, 4000, 12500>
    colour 2*White
    media_interaction off
  }
  
  union {
    sphere { <0, 0, 0>, 2500
      texture {
        T_Planet
        scale 2500
      }
    }
    // rings
    difference {
      cylinder { <0, -10, 0>, <0, 10, 0>, 5000 }
      cylinder { <0, -11, 0>, <0, 11, 0>, 3500 }
      pigment {
        cylindrical
        turbulence 0.02
        colour_map {
          [ 0.00 colour rgbt <0.00, 0.00, 0.00, 1.00> ]
          [ 0.10 colour rgbt <0.00, 1.00, 1.20, 1.00> ]
          [ 0.13 colour rgbt <0.60, 1.90, 2.40, 0.50> ]
          [ 0.17 colour rgbt <0.40, 1.30, 1.70, 0.65> ]
          [ 0.19 colour rgbt <0.30, 1.44, 1.66, 0.75> ]
          [ 0.22 colour rgbt <0.20, 2.00, 1.60, 0.50> ]
          [ 0.25 colour rgbt <0.40, 2.00, 1.40, 0.65> ]
          [ 0.27 colour rgbt <0.70, 1.30, 1.40, 0.75> ]
          [ 0.29 colour rgbt <1.00, 0.60, 1.40, 0.70> ]
          [ 0.31 colour rgbt <0.00, 2.00, 2.00, 0.50> ]
          [ 0.34 colour rgbt <0.60, 1.80, 1.80, 0.55> ]
          [ 0.37 colour rgbt <0.00, 1.00, 1.00, 0.70> ]
          [ 1.00 colour rgbt <0.00, 0.00, 0.00, 1.00> ]
        }
        scale 5000/0.9
      }
      finish {
        ambient 0//.5
        //diffuse 1
      }         
    }
    no_shadow
    rotate <-15, 0, 25>
    translate <7000, 2850, 20000>
    pigment { colour Cyan }
    photons { collect off }
  }
  
  global_lights off
}

// *** End Planet Stuff *** \\

// *** Begin Crystal Stuff *** \\
#if ( crystals )

  // base for crystals
  #declare holder_major = 0.8;
  #declare holder_minor = 0.15;
  #declare Holder = union {
    torus { holder_major, holder_minor }
    cylinder { <0, -100, holder_major>, <0, 0, holder_major>, holder_minor }
    cylinder { <0, -100, holder_major>, <0, 0, holder_major>, holder_minor
      rotate y*120
    }
    cylinder { <0, -100, holder_major>, <0, 0, holder_major>, holder_minor
      rotate y*240
    }
    texture { T_Silver_4B }
  }
  
  #macro Crystal_mat ( tint )
    // select colour of crystal
    // yellow = <1.00, 1.00, 0.80>
    // blue = <0.75, 0.85, 1.00>
    // pink = <1.00, 0.80, 1.00>
    // red = <1.20, 0.75, 0.75>
    // green = <0.80, 1.00, 0.80>
    #switch ( tint )
      #range ( 0/5, 1/5 )
        #local tint_interior = colour rgbft <1.00, 1.00, 0.80, 0.15, 0.05>;
        #local tint_exterior = colour rgbft <1.00, 1.00, 0.80, 0.30, 0.15>;
      #break
      #range ( 1/5, 2/5 )
        #local tint_interior = colour rgbft <0.75, 0.85, 1.00, 0.15, 0.05>;
        #local tint_exterior = colour rgbft <0.75, 0.85, 1.00, 0.30, 0.15>;
      #break
      #range ( 2/5, 3/5 )
        #local tint_interior = colour rgbft <1.00, 0.80, 1.00, 0.15, 0.05>;
        #local tint_exterior = colour rgbft <1.00, 0.80, 1.00, 0.30, 0.15>;
      #break
      #range ( 3/5, 4/5 )
        #local tint_interior = colour rgbft <1.20, 0.75, 0.75, 0.15, 0.05>;
        #local tint_exterior = colour rgbft <1.20, 0.75, 0.75, 0.30, 0.15>;
      #break
      #else
        #local tint_interior = colour rgbft <0.75, 1.20, 0.75, 0.15, 0.05>;
        #local tint_exterior = colour rgbft <0.75, 1.20, 0.75, 0.30, 0.15>;
    #end
    material {
      texture {
        pigment {
          spherical
          colour_map {
            #if ( crystal_detail )
              [0 colour tint_exterior]
              [1 colour tint_interior]
            #else
              [0 colour tint_exterior - <0, 0, 0, 0.30, 0.15>]
              [1 colour tint_interior - <0, 0, 0, 0.15, 0.05>]
            #end
          }
        }
        finish {
          #if ( crystal_detail )
            reflection 0.2
            phong 0.2
            metallic
          #end
          ambient 0.6
        }
      }
      #if ( crystal_detail )
        interior {
          ior 1.5
          //dispersion 1.1
        }
      #end
    }
  #end

  #declare spiral_radius = 5*2;
  #declare sphere_radius = 0.2*5;
  #declare angle_up = 2*0.75;
  #declare angle_max = 90;
  #declare angle_in = 2;
  #declare spheres = 0;
  #declare min_distance = 2;
  #declare Top_Location = <0, 0, 0>;
  #declare Crystal_Colours = array[6][3]
  {
    {0.3, 0.5, 0.9},
    {0.3, 0.9, 0.5},
    {0.5, 0.3, 0.9},
    {0.5, 0.9, 0.3},
    {0.9, 0.3, 0.5},
    {0.9, 0.5, 0.3}
  }
  #declare Crystal_Rand = seed(4526);

  #macro Crystal_Spiral( light_quality, rotation, translation )
    // colours
    #local Media_Base = colour rgb <0.4, 0.6, 0.6>;
    // random function for uniform distribution of integers in the range [0, 5]
    #local Colour_Index = 1;
    #while ( Colour_Index = 1 )
      #local Colour_Index = rand( Crystal_Rand );
    #end
    #local Colour_Index = floor( Colour_Index*6 );

    // spiral
    union {
      // the first colour is fixed so permutations which are rotations of each other are not considered
      Spiral ( spiral_radius, sphere_radius, 0, angle_up, angle_max, angle_in, spheres, min_distance, Crystal_mat(0.1), Top_Location )
      object { Holder
        translate <0, -sqrt( pow( sphere_radius + holder_minor, 2 ) - pow( holder_major, 2 ) ), spiral_radius>
      }
      Spiral ( spiral_radius, sphere_radius, 90, angle_up, angle_max, angle_in, spheres, min_distance, Crystal_mat( Crystal_Colours[Colour_Index][0] ), Top_Location )
      object { Holder
        translate <0, -sqrt( pow( sphere_radius + holder_minor, 2 ) - pow( holder_major, 2 ) ), spiral_radius>
        rotate y*90
      }
      Spiral ( spiral_radius, sphere_radius, 180, angle_up, angle_max, angle_in, spheres, min_distance, Crystal_mat( Crystal_Colours[Colour_Index][1] ), Top_Location )
      object { Holder
        translate <0, -sqrt( pow( sphere_radius + holder_minor, 2 ) - pow( holder_major, 2 ) ), spiral_radius>
        rotate y*180
      }
      Spiral ( spiral_radius, sphere_radius, 270, angle_up, angle_max, angle_in, spheres, min_distance, Crystal_mat( Crystal_Colours[Colour_Index][2] ), Top_Location )
      object { Holder
        translate <0, -sqrt( pow( sphere_radius + holder_minor, 2 ) - pow( holder_major, 2 ) ), spiral_radius>
        rotate y*270
      }
      // holder for pearl ball
      #declare Leg = union {
        cylinder { <0, -100, 0>, <0, 0, 0>, 0.3 }
        sphere_sweep {
          b_spline
          1600,
          #declare i = 0;
          #while ( i < 1600 )
            <0.3*cos( i*pi/4 ), -100 + i/16, 0.3*sin( i*pi/4 )>, 0.1
            #declare i = i+1;
          #end
          texture { T_Gold_4C }
        }
      }
      union {
        torus { 3, 0.3 }
        sphere_sweep {
          b_spline
          130,
          #declare i = 0;
          #while ( i < 130 )
            vrotate( <0, 0.3*cos( i*pi/4 ), 3+0.3*sin( i*pi/4 )>, y*360/128*i ), 0.1
            #declare i = i + 1;
          #end
          texture { T_Gold_4C }
        }
        object { Leg
          translate <0, 0, 3>
        }
        object { Leg
          translate <3, 0, 0>
        }
        object { Leg
          translate <0, 0, -3>
        }
        object { Leg
          translate <-3, 0, 0>
        }
        texture { T_Silver_4C }
        translate <0, 5 - sqrt( pow( 3.4, 2 ) - pow( 3, 2 ) ), 0>
      }
      // pearl ball
      sphere { <0, 0, 0>, 1
        material {
          texture {
            pigment {
              spherical
              colour_map {
                [0 colour rgbft <1.0, 1.0, 1.0, 0.30, 0.15>]
                [1 colour rgbft <1.0, 1.0, 1.0, 0.15, 0.05>]
              }
            }
            finish {
              reflection 0.2
              phong 0.2
              metallic
              ambient 0.6
            }
          }
          interior {
            ior 1.2
            //dispersion 1.1
          }
        }
        scale 3
        translate y*5
      }
      #if ( media_effects )
        // glow around ball
        merge {
          sphere { <0, 0, 0>, 1 }
          difference {
            sphere { <0, 0, 0>, 1
              scale <1, 1.25, 1>
            }
            plane { y, 0 }
          }
          hollow
          no_shadow
          pigment { colour rgbt <1, 1, 1, 1> }
          interior {
            media {
              emission 1
              intervals 5
              density {
                spherical
                colour_map {
                  [ 0.00 colour rgb <0, 0, 0> ]
                  [ 0.02 colour Media_Base*0.2 ]
                  [ 1.00 colour Media_Base*0.2 ]
                }
                scale <1, 1.25, 1>
              }
            }
          }
          scale 3.75
          translate y*5
        }
        // beam
        cylinder { <0, 0, 0>, <0, 10*Top_Location.y, 0>, 1
          hollow
          no_shadow
          pigment { colour rgbt <1, 1, 1, 1> }
          interior {
            media {
              emission 1
              intervals 5
              density {
                cylindrical
                turbulence 0.1
                colour_map {
                  [ 0.00 colour rgb <0, 0, 0> ]
                  [ 0.70 colour rgb Media_Base*0.5 ]
                  [ 1.00 colour rgb Media_Base*0.5 ]
                }
              }
            }
          }
          scale 2.5
          translate y*8
        }
      #end
      #if ( photon )
        photons {
          target
          reflection on
          refraction on
          collect off
        }
      #end
      rotate rotation
      translate translation + y
    }
    #if ( crystal_light )
      #if ( true | light_quality )
        light_source {
          translation + <0, (Top_Location.y)/2, 0>,
          colour rgb <1, 1, 1>
          fade_distance (Top_Location.y)*0.65
          fade_power 2.5
          area_light
          x, (Top_Location.y)*y*7/8, 1, 20
          jitter
          adaptive 1
          media_interaction off
        }
      #else
        #local i = Top_Location.y/6;
        #while ( i < Top_Location.y )
          light_source {
            translation + <0, i, 0>,
            colour rgb <1, 1, 1>
            fade_distance ( spiral_radius*(1 - pow( (i/Top_Location.y), 2 ) ) + 1 )/1.25
            fade_power 1.25
            media_interaction off
          }
          #local i = i + Top_Location.y/6;
        #end
      #end
    #end
  #end

#end
// *** End Crystal Stuff *** \\

// *** Floating Cages *** \\
#if ( cages )
  #macro Lissajous_Sphere ( Rsphere, Rmajor, Rminor, Spirals, Revolutions, Fcos, Fsin, Strands, Textures )
    #local T_rev = 1/(Fsin*Fcos);
    #local delta_T = 1/8*min(1/Fsin, 1/Fcos);
    #local N = ceil(Revolutions*T_rev/delta_T);
    #local Ry = 360 * Spirals;
    #local Fsphere = 2*pi/N;
    union {
      //#local delta_phi = 2*pi*T_rev/Strands;
      #local delta_phi = 2*pi/Strands;
      #local j = 0;
      #while ( j < Strands )
        #local phi = j*delta_phi;
        sphere_sweep {
          b_spline
          N + 3,
          #local i = 0;
          #while ( i < N + 3 )
            #local T = delta_T*i;
            #if ( Fsphere*i <= pi )
              /*
              #if ( Fsphere*i <= pi/2 )
                #local Rz = degrees(Fsphere*i)*( 0.5*sin(Fsphere*i) );
                #local i_last1 = i;
              #else
                #local Rz = degrees(Fsphere*i)*( 0.5*sin(Fsphere*i_last1) + (-0.5*cos(Fsphere*(i-i_last1)) + 0.5) );
                #local i_last2 = i;
              #end
              */
              #local Rotate = Ry*(-0.5*cos(Fsphere*i) + 0.5);
              #local i_last = i;
            #else
              #local Rotate = Ry*(-0.5*cos(Fsphere*i_last) + 0.5) - Ry*(-0.5*cos(Fsphere*(i-i_last)) + 0.5);
              /*
              #if ( Fsphere*i <= 3*pi/2 )
                #local Rz = degrees(Fsphere*i)*( 0.5*sin(Fsphere*i_last1) + (-0.5*cos(Fsphere*(i_last2-i_last1)) + 0.5) + 0.5*sin(Fsphere*(i-i_last2)) );
                #local i_last3 = i;
              #else
                #local Rz = degrees(Fsphere*i)*( 0.5*sin(Fsphere*i_last1) + (-0.5*cos(Fsphere*(i_last2-i_last1)) + 0.5) + 0.5*sin(Fsphere*(i_last3-i_last2)) + (-0.5*cos(Fsphere*(i-i_last3)) + 0.5) );
              #end
              */
            #end
            #local Pos = <0, Rmajor*cos(2*pi*Fcos*T + phi)+Rsphere, Rmajor*sin(2*pi*Fsin*T + phi)>;
            #local Pos = vrotate( Pos, y*Rotate );
            #local Pos = vrotate( Pos, -z*degrees(Fsphere*i) );
            #local Pos = vrotate( Pos, y*Rotate );
            Pos, Rminor
            //vrotate( vrotate( <0, Rmajor*cos(2*pi*Fcos*T + phi)+Rsphere, Rmajor*sin(2*pi*Fsin*T + phi)>, -z*degrees(Fsphere*i) ), y*Rotate ), Rminor
            #local i = i + 1;
          #end
          texture { Textures[j] }
        }
        #local j = j + 1;
      #end
      bounded_by { sphere { <0, 0, 0>, Rsphere + Rmajor + Rminor } }
    }
  #end

  #macro Cage ( rotation, translation, light_quality )
    // light group if light_quality is false
    #if ( !light_quality & cage_lights )
      light_group {
    #end
      union {
        #if ( cage_detail )
          #declare LS_Textures = array[2] { T_Silver_4C, T_Gold_4C };
        #else
          #declare LS_Textures = array[2] {
            texture { pigment { colour rgb <0.95, 0.91, 0.91> } finish { ambient 0.3 } },
            texture { pigment { colour rgb <0.9, 0.9, 0.9> } finish { ambient 0.3 } }
          }
        #end
        Lissajous_Sphere ( 10, 2, 0.2, 2, 20, 2, 1, 2, LS_Textures )
        #if ( media_effects )
          sphere { <0, 0, 0>, 1
            hollow
            no_shadow
            pigment { colour rgbt <1, 1, 1, 1> }
            interior {
              media {
                emission 1
                intervals 10
                density {
                  spherical
                  turbulence 0.2
                  colour_map {
                    [ 0.00 colour rgb <0, 0, 0> ]
                    [ 0.25 colour <0.18, 0.7, 0.18>*0.1 ]
                    [ 1.00 colour <0.35, 1.5, 0.35>*0.1 ]
                  }
                }
              }
            }
            scale 9
          }
        #end
        rotate rotation
        translate translation
      }
      #if ( cage_lights )
        #if ( light_quality )
          light_source {
            translation
            colour rgb <0.5, 1, 0.5>*0.3
            fade_distance 15
            fade_power 3
            media_interaction off
          }
        #else
          light_source {
            translation
            colour rgb <0.5, 1, 0.5>*0.5
            media_interaction off
          }
          global_lights off
        }
      #end
    #end
  #end
  
  #declare Cage_Rand = seed(534844);
  
  #declare Cage_Loc = array[20]
  
  #declare j = 0;
  #while ( j < 20 )
    #declare Pos = <0, 50+350*pow( rand(Cage_Rand), 2 ), 100+900*rand(Cage_Rand)>;
    #declare Pos = vrotate( Pos, y*(60*rand(Cage_Rand) - 30) );
    // check if too close to ground
    #declare Surface = trace( Terrain, <Pos.x, 500, Pos.y>, -y );
    #declare Dist_to_surface = Pos.y - Surface.y;
    #if ( Dist_to_surface < 25 )
      #declare okay = false;
    #else
      #declare okay = true;
    #end
    // check if too close to a Spiral
    #declare k = 0;
    #while ( k < 6 & okay )
      #if ( VDist( Spiral_Loc[k], <Pos.x, 0, Pos.z> ) < 20 )
        #declare okay = false;
      #end
      #declare k = k + 1;
    #end
    // check if too close to another cage
    #declare k = 0;
    #while ( k < j & okay )
      #if ( VDist( Cage_Loc[k], Pos ) < 30 )
        #declare okay = false;
      #end
      #declare k = k + 1;
    #end
    #if ( okay )
      Cage( <360*rand(Cage_Rand), 360*rand(Cage_Rand), 360*rand(Cage_Rand)>, Pos, false )
      #declare Cage_Loc[j] = Pos;
      #declare j = j + 1;
    #end
  #end
  Cage( <360*rand(Cage_Rand), 360*rand(Cage_Rand), 360*rand(Cage_Rand)>, <-50, 50, 150>, false )
  Cage( <720*rand(Cage_Rand), 720*rand(Cage_Rand), 720*rand(Cage_Rand)>, <100, 50, 175>, false )
#end
// *** End Floating Cages *** \\

// *** Mushrooms *** \\
#declare MRand = seed(209357);
#macro Mushroom( Normal )
  merge {
    cone { -0.5*Normal, 0.3, Normal, 0.2 }
    #local MR_top = blob {
      threshold 0.5
      sphere { <0, 0, 0>, 1, 1 }
      sphere { <0, -1, 0>, 1, -1 }
      #local i = 0;
      #while ( i < 5 )
        #local Pos = <0, 0, 0.4>;
        #local Pos = vrotate( Pos, -x*180*rand(MRand) );
        #local Pos = vrotate( Pos, y*360*rand(MRand) );
        sphere { Pos, 0.1 + 0.4*rand(MRand), 0.3*(rand(MRand) - 0.25) }
        #local i = i + 1;
      #end
      scale 1.25
    }
    #local top = trace( MR_top, 10*y, -y );
    #local bottom = trace ( MR_top, -10*y, y );
    object { MR_top    
      translate -(top + bottom)/2
      #local Rx = -degrees( atan2( Normal.z, Normal.y ) );
      #local Rz = -degrees( atan2( Normal.x, Normal.y ) );
      rotate <Rx, 0, Rz>
      translate Normal
    }
  }
#end    

#declare T_Mushroom = texture {
  pigment {
    bozo
    turbulence 0.25
    octaves 4
    omega 0.7
    colour_map {
      /*
      [ 0.00 colour rgb <0.8, 0.8, 1.0> ]
      [ 0.20 colour rgb <0.8, 0.8, 1.0> ]
      [ 0.35 colour rgb <0.5, 0.5, 1.0> ]
      [ 0.65 colour rgb <0.5, 0.5, 1.0> ]
      [ 0.80 colour rgb <0.0, 0.0, 0.7> ]
      [ 1.00 colour rgb <0.0, 0.0, 0.7> ]
      */
      [ 0.00 colour rgbf <0.8, 0.8, 1.0, 0.4> ]
      [ 0.20 colour rgbf <0.8, 0.8, 1.0, 0.4> ]
      [ 0.35 colour rgbf <0.5, 0.5, 1.0, 0.3> ]
      [ 0.65 colour rgbf <0.5, 0.5, 1.0, 0.3> ]
      [ 0.80 colour rgbf <0.0, 0.0, 0.7, 0.1> ]
      [ 1.00 colour rgbf <0.0, 0.0, 0.7, 0.1> ]
    }
    scale 0.35
  }
  //finish { ambient 0.2 }
}
// *** End Mushroom Stuff *** \\

// *** Light Group Scene Assembly *** \\
#if ( crystals )
  // left back
  light_group {
    #declare Surface = trace( Terrain, <-400, 500, 800>, -y );
    #declare max_y = Surface.y;
    #declare Surface = trace( Terrain, <-400+spiral_radius, 500, 800>, -y );
    #if ( Surface.y > max_y )
      #declare max_y = Surface.y;
    #end
    #declare Surface = trace( Terrain, <-400-spiral_radius, 500, 800>, -y );
    #if ( Surface.y > max_y )
      #declare max_y = Surface.y;
    #end
    #declare Surface = trace( Terrain, <-400, 500, 800+spiral_radius>, -y );
    #if ( Surface.y > max_y )
      #declare max_y = Surface.y;
    #end
    #declare Surface = trace( Terrain, <-400, 500, 800-spiral_radius>, -y );
    #if ( Surface.y > max_y )
      #declare max_y = Surface.y;
    #end
    Crystal_Spiral( false, 0, <-400, max_y, 800> )
    // back  
    light_group {
      #declare Surface = trace( Terrain, <-100, 500, 750>, -y );
      #declare max_y = Surface.y;
      #declare Surface = trace( Terrain, <-100+spiral_radius, 500, 750>, -y );
      #if ( Surface.y > max_y )
        #declare max_y = Surface.y;
      #end
      #declare Surface = trace( Terrain, <-100-spiral_radius, 500, 750>, -y );
      #if ( Surface.y > max_y )
        #declare max_y = Surface.y;
      #end
      #declare Surface = trace( Terrain, <-100, 500, 750+spiral_radius>, -y );
      #if ( Surface.y > max_y )
        #declare max_y = Surface.y;
      #end
      #declare Surface = trace( Terrain, <-100, 500, 750-spiral_radius>, -y );
      #if ( Surface.y > max_y )
        #declare max_y = Surface.y;
      #end
      Crystal_Spiral( false, 0, <-100, max_y, 750> )
      // right far
      light_group {
        #declare Surface = trace( Terrain, <200, 500, 400>, -y );
        #declare max_y = Surface.y;
        #declare Surface = trace( Terrain, <200+spiral_radius, 500, 400>, -y );
        #if ( Surface.y > max_y )
          #declare max_y = Surface.y;
        #end
        #declare Surface = trace( Terrain, <200-spiral_radius, 500, 400>, -y );
        #if ( Surface.y > max_y )
          #declare max_y = Surface.y;
        #end
        #declare Surface = trace( Terrain, <200, 500, 400+spiral_radius>, -y );
        #if ( Surface.y > max_y )
          #declare max_y = Surface.y;
        #end
        #declare Surface = trace( Terrain, <200, 500, 400-spiral_radius>, -y );
        #if ( Surface.y > max_y )
          #declare max_y = Surface.y;
        #end
        Crystal_Spiral( false, 0, <200, max_y, 400> )
        // right
        light_group {
          #declare Surface = trace( Terrain, <75, 500, 300>, -y );
          #declare max_y = Surface.y;
          #declare Surface = trace( Terrain, <75+spiral_radius, 500, 300>, -y );
          #if ( Surface.y > max_y )
            #declare max_y = Surface.y;
          #end
          #declare Surface = trace( Terrain, <75-spiral_radius, 500, 300>, -y );
          #if ( Surface.y > max_y )
            #declare max_y = Surface.y;
          #end
          #declare Surface = trace( Terrain, <75, 500, 300+spiral_radius>, -y );
          #if ( Surface.y > max_y )
            #declare max_y = Surface.y;
          #end
          #declare Surface = trace( Terrain, <75, 500, 300-spiral_radius>, -y );
          #if ( Surface.y > max_y )
            #declare max_y = Surface.y;
          #end
          Crystal_Spiral( false, 0, <75, max_y, 300> )
          // left
          light_group {
            #declare Surface = trace( Terrain, <-100, 500, 250>, -y );
            #declare max_y = Surface.y;
            #declare Surface = trace( Terrain, <-100+spiral_radius, 500, 250>, -y );
            #if ( Surface.y > max_y )
              #declare max_y = Surface.y;
            #end
            #declare Surface = trace( Terrain, <-100-spiral_radius, 500, 250>, -y );
            #if ( Surface.y > max_y )
              #declare max_y = Surface.y;
            #end
            #declare Surface = trace( Terrain, <-100, 500, 250+spiral_radius>, -y );
            #if ( Surface.y > max_y )
              #declare max_y = Surface.y;
            #end
            #declare Surface = trace( Terrain, <-100, 500, 250-spiral_radius>, -y );
            #if ( Surface.y > max_y )
              #declare max_y = Surface.y;
            #end
            Crystal_Spiral( false, 0, <-100, max_y, 250> )
            // front centre
            light_group {
              #declare Surface = trace( Terrain, <1, 500, 110>, -y );
              #declare max_y = Surface.y;
              #declare Surface = trace( Terrain, <1+spiral_radius, 500, 110>, -y );
              #if ( Surface.y > max_y )
                #declare max_y = Surface.y;
              #end
              #declare Surface = trace( Terrain, <1-spiral_radius, 500, 110>, -y );
              #if ( Surface.y > max_y )
                #declare max_y = Surface.y;
              #end
              #declare Surface = trace( Terrain, <1, 500, 110+spiral_radius>, -y );
              #if ( Surface.y > max_y )
                #declare max_y = Surface.y;
              #end
              #declare Surface = trace( Terrain, <1, 500, 110-spiral_radius>, -y );
              #if ( Surface.y > max_y )
                #declare max_y = Surface.y;
              #end
              Crystal_Spiral( true, 0, <1, max_y, 110> )
#end
#if ( cages )
              light_group {
                Cage( <360*rand(Cage_Rand), 360*rand(Cage_Rand), 360*rand(Cage_Rand)>, <22, 30, 55>, true )
                light_group {
                  Cage( <360*rand(Cage_Rand), 360*rand(Cage_Rand), 360*rand(Cage_Rand)>, <-30, 30, 72>, true )
#end
                  light_group {
                    // cut to allow light grouping
                    //difference {
                    object { Terrain //}
                    /*  cylinder { <1, -75.1, 100>, <1, 312.6, 110>, 80 }
                      cylinder { <-100, -75.1, 750>, <-100, 312.6, 750>, 80 }
                      cylinder { <200, -75.1, 400>, <200, 312.6, 400>, 80 }
                      cylinder { <75, -75.1, 300>, <75, 312.6, 300>, 80 }
                      cylinder { <-100, -75.1, 250>, <-100, 312.6, 250>, 80 }
                      cylinder { <-400, -75.1, 800>, <-400, 312.6, 800>, 80 }*/
                      #if ( hf_detail )
                        texture { T_Terrain }
                      #else
                        pigment { colour rgb <1, 0, 1> }
                        finish { ambient 0.2 }
                      #end
                    }
                    // water
                    plane { y, -12.5
                      #if ( water_detail )
                        material { Water }
                      #else
                        pigment { colour rgb <0, 0, 1> }
                      #end
                    }
                    //----- Rocks -----\\
                    #if ( rocks )
                      #declare Rock_Rand = seed(734);  // random seed for rock placement and sizing
                      #declare Nrocks = 70;
                      #declare Rock_Loc = array[Nrocks];
                      #declare Rock_radius = array[Nrocks];
                      #declare i = 0;
                      #while ( i < Nrocks )
                      	// generate random x-z plane locaton and radius
                        #declare Pos = <0, 0, 750*rand(Rock_Rand) - 200>;
                        #declare Pos = vrotate( Pos, y*(60*rand(Rock_Rand) - 30) );
                      	#declare rad = 0.5 + 0.75*rand(Rock_Rand);
                      	// calculate y position using trace function, and move down a bit (based on radius)
                      	#declare Norm = <0, 0, 0>;
                      	#declare Pos = trace( Terrain, <Pos.x, 500, Pos.z>, -y, Norm ) - y*0.25*rad*( 0.1 + rand(Rock_Rand) );
                      	// check slope at intersection point
                      	#declare Slope = vdot( Norm, y );
                      	#if ( Slope < 0.95 )
                      	  #declare okay = false;
                      	#else
                      	  #declare okay = true;
                      	#end
                        // check if too close to a Spiral
                        #declare j = 0;
                        #while ( j < 6 & okay )
                          #if ( VDist( Spiral_Loc[j], <Pos.x, 0, Pos.z> ) < 12 )
                            #declare okay = false;
                          #end
                          #declare j = j + 1;
                        #end
                        #if ( okay )
                          // Extract xyz positions (the f_sphere call doesn't like Pos.x for some reason)
                        	#declare xpos = Pos.x;
                        	#declare ypos = Pos.y;
                        	#declare zpos = Pos.z;
                        	// calculate "scale factors" for axes in sphere function
                        	#declare xscale = 0.5 + rand(Rock_Rand);
                        	#declare yscale = 0.5 + rand(Rock_Rand);
                        	#declare zscale = 0.5 + rand(Rock_Rand);
                        	// generate random noise strength
                        	#declare noise = 0.3 + 0.1*rand(Rock_Rand);
                        	// rock object
                        	isosurface {
                        		function {
                        			// scaled sphere function, centred at random location
                              f_sphere( xscale*(x-xpos), yscale*(y-ypos), zscale*(z-zpos), rad )
                        			// noise function to affect shape
                        			+ noise*rad*fn_Pigm( 0.1*x, 0.1*y, 0.1*z ).gray
                        			// finer noise function for more detailed surface
                        			+ (noise/2)*rad*fn_Pigm( 2*(x + 168), 2*(y + 882), 2*(z + 308) ).gray
                        		}
                        		contained_by { sphere { Pos, 1.5*rad }	}
                        	  accuracy 0.001
                        	  max_gradient 10
                        	  #if ( rock_detail )
                        	  	texture { T_Rock }
                        	  #else
                        	  	pigment { colour rgb <0.5, 0.5, 0.5> }
                        	  #end
                        	}
                        	#declare Rock_Loc[i] = Pos;
                        	#declare Rock_radius[i] = 1.5*rad;
                        	#declare i = i + 1;
                        #end
                      #end
                    #end
                    //----- End Rocks -----\\
                    
                    //----- Mushrooms -----\\
                    #if ( mushrooms )
                      #declare Mushroom_Rand = seed(791218);  // random seed for rock placement and sizing
                      #declare mindist = 12;
                      #declare maxdist = 75;
                      #declare Ntries = 150;
                      #declare M_Loc = array[Ntries];
                      #declare M_radius = array[Ntries];
                      // for each spiral
                      #declare i = 0;
                      #while ( i < 6 )
                        #declare j = 0;
                        #declare Nplaced = 0;
                        #while ( j < Ntries )
                        	// generate random distance
                          #declare dist = mindist + (maxdist - mindist)*rand(Mushroom_Rand);
                          // probability of mushroom existance decays as you get farther away from the spiral
                          #if ( mindist + (maxdist - mindist)*rand(Mushroom_Rand) >= dist )
                            // generate coordinate in x-z plane
                            #declare Pos = vrotate( <0, 0, dist>, y*360*rand(Mushroom_Rand) ) + Spiral_Loc[i];
                            // calculate y position using trace funtion
                            #declare Norm = <0, 0, 0>;
                            #declare Pos = trace( Terrain, <Pos.x, 500, Pos.z>, -y, Norm );
                            // check slope at intersection point
                            #declare Slope = vdot( Norm, y );
                            #if ( Slope < 0.85 )
                              #declare okay = false;
                            #else
                              #declare okay = true;
                            #end
                            // compute a scale factor (based on distance from sprials)
                            #declare Mscale = (maxdist - dist/1.5)/maxdist * ( 1.2 + 0.4*rand(Mushroom_Rand) );
                            // collision detection with other mushrooms
                            #declare k = 0;
                            #while ( okay & k < Nplaced )
                              #if ( VDist( Pos, M_Loc[k] ) < M_radius[k] + 0.5*Mscale )
                                #declare okay = false;
                              #end
                              #declare k = k + 1;
                            #end
                            // collision detection with rocks
                            #if ( rocks )
                              #declare k = 0;
                              #while ( okay & k < Nrocks )
                                #if ( VDist( Pos, Rock_Loc[k] ) < Rock_radius[k] + 0.5*Mscale )
                                  #declare okay = false;
                                #end
                                #declare k = k + 1;
                              #end
                            #end
                            // place if okay
                            #if ( okay )
                              object {
                                Mushroom( Norm )
                                scale Mscale
                                translate Pos
                                texture { T_Mushroom }
                              }
                              #declare M_Loc[Nplaced] = Pos;
                              #declare M_radius[Nplaced] = 0.5*Mscale;
                              #declare Nplaced = Nplaced + 1;
                            #end
                          #end
                          #declare j = j + 1;
                        #end
                        #declare i = i + 1;
                      #end
                    #end
                          
                    // moonlight
                    light_source {
                      <2750, 2250, 10000>
                      colour rgb <0.3, 0.7, 0.7>
                      media_interaction off
                    }
                    #if ( test_light )
                      light_source {
                        <-150, 500, -100>
                        colour White
                        #if ( lighting_detail )
                          area_light
                          25*x, 25*y, 3, 3
                          jitter
                          adaptive 0.5
                        #end
                        media_interaction off
                      }
                    #end
                  }
#if ( cages )
                }
              }
#end
#if ( crystals )
            }
          }
        }
      }
    }
    global_lights on
  }
#end
