// Persistence of Vision Ray Tracer Scene Description File
// File: spiral.inc
// Vers: 3.6
// Desc: Spiral Macro
// Date: August 26th, 2004
// Auth: Philip Chan

#macro Spiral ( Sradius, Cradius, Angle_Start, AU_factor, AU_max, Angle_in, N, Tfactor, Material, Top_location )
  #local Current = <Sradius, 0, 0>;
  #local Count = 0;
  #local Angle_up = 0;
  union {
    #while ( vlength(<Current.x, 0, Current.z>) > Tfactor*Cradius & (!N | Count < N) & (Angle_up <= AU_max ) )
      sphere { <0, 0, 0>, 1
        material { Material }
        scale Cradius
        translate Current
      }
      // calculate angle up and increment counter
      //#local Angle_up = Angle_up + Count*AU_growth;
      #local Count = Count + 1;
      // rotate to next spot on circle of current radius
      #local Rotate_y = degrees( 2*asin( Cradius/vlength(<Current.x, 0, Current.z>) ) );
      #local Angle_up = Rotate_y * AU_factor;
      #local Next = vrotate(Current, y*Rotate_y);
      // calculate vector from current position to next position, rotate to form inward spiral
      #local To_Next = vrotate( Next - Current, y*Angle_in );
      // calculate rotation axis perpendicular to y and To_Next
      #local R_axis = vnormalize( vrotate( To_Next, -y*90 ) );
      // rotate To_Next vector up to form upward spiral
      #local To_Next = vrotate( To_Next, R_axis*Angle_up );
      // set new current position
      #local Current = Current + To_Next;
    #end
    rotate y*Angle_Start
  }
  #declare Top_location = Current;
#end
