// Persistence of Vision Ray Tracer Scene Description File
// File: terrain_hf.pov
// Vers: 3.6
// Desc: Generates the height field targa file for povcomp2004.pov
// Date: October 3rd, 2004
// Auth: Philip Chan

// NOTE:  MUST be rendered with display gamma set to 2.5 to get the same results as original.

// test render
// +FT +W750 +750
// final render (resolution can be increased for better quality)
// +FT +A0.1 +W5000 +H5000 -D

global_settings {
	assumed_gamma 2.2
	hf_gray_16
}

camera {
	up y
	right x
	location <0, 30, 0>
	look_at <0, 0, 0>
}

#declare Rolling_Hills = texture {
  pigment {
  	bozo
  	color_map {
  		[0.00 colour rgb <0.2, 0.2, 0.2>]
  		[1.00 colour rgb <0.26, 0.26, 0.26>]
  	}
  	scale <1.15, 0.7, 0.7>
  }
  finish { ambient 1 }
}

#declare Tiny_Bumps = texture {
  pigment {
    bozo
    colour_map {
      [0.00 colour rgbt <0.23, 0.23, 0.23, 1.0>]
      [0.50 colour rgbt <0.23, 0.23, 0.23, 1.0>]
      [1.00 colour rgbt <0.23, 0.23, 0.23, 0.9>]
    }
    scale 1/75
  }
  finish { ambient 1 }
}

#declare Rough_Hills = texture {
  pigment {
    bozo
    turbulence 0.5
    omega 0.75
    lambda 1.25
    colour_map {
      [0.00 colour rgbt <0.29, 0.29, 0.29, 1.0>]
      [0.50 colour rgbt <0.29, 0.29, 0.29, 1.0>]
      [1.00 colour rgbt <0.29, 0.29, 0.29, 0.5>]
    }
    scale 1
  }
  finish { ambient 1 }
}
#declare Rough_Hills_Transparent = texture {
  pigment { colour rgbt <0.31, 0.31, 0.31, 1.0> }
}
#declare SRH_scale = 40;
#declare Sloped_Rough_Hills = texture {
  gradient z
  turbulence 0.1
  texture_map {
    [0.00 Rough_Hills_Transparent]
    [0.20 Rough_Hills_Transparent]
    [0.40 Rough_Hills scale 1/SRH_scale]
    [1.00 Rough_Hills scale 1/SRH_scale]
  }
  scale SRH_scale
}

#declare Tiny_Cracks = texture {
  pigment {
    crackle
    turbulence 0.3
    colour_map {
      [0.0000 colour rgbt <0.2, 0.2, 0.2, 0.7>]
      [0.0020 colour rgbt <0.2, 0.2, 0.2, 0.7>]
      [0.0025 colour rgbt <0.2, 0.2, 0.2, 1.0>]
      [1.0000 colour rgbt <0.2, 0.2, 0.2, 1.0>]
    }
    scale 1
  }
  finish { ambient 1 }
}
  
plane { y, 0
  texture { Rolling_Hills }
  texture { Tiny_Bumps }
  texture { Rough_Hills }
//  texture { Tiny_Cracks }
}

// terraforming

// central mountain
disc { <0, 0, 0>, y, 1
  pigment {
    spherical
    turbulence 0.2
    colour_map {
      [0.00 colour rgbt <1.0, 1.0, 1.0, 1.0>]
      [0.75 colour rgbt <1.0, 1.0, 1.0, 0.7>]
      [1.00 colour rgbt <1.0, 1.0, 1.0, 0.0>]
    }
    scale 0.75
  }
  finish { ambient 1 }
  scale <16, 1, 12>
  translate <-10, 0.001, 10>
}
// left front mountain
disc { <0, 0, 0>, y, 1
  pigment {
    spherical
    turbulence 0.2
    colour_map {
      [0.00 colour rgbt <0.80, 0.80, 0.80, 1.0>]
      [0.50 colour rgbt <0.80, 0.80, 0.80, 0.7>]
      [1.00 colour rgbt <0.80, 0.80, 0.80, 0.0>]
    }
    scale 0.75
  }
  finish { ambient 1 }
  scale <6, 1, 4>
  translate <-12, 0.002, 8>
}
// left mountain
disc { <0, 0, 0>, y, 1
  pigment {
    spherical
    turbulence 0.2
    colour_map {
      [0.00 colour rgbt <0.92, 0.92, 0.92, 1.0>]
      [0.50 colour rgbt <0.92, 0.92, 0.92, 0.7>]
      [1.00 colour rgbt <0.92, 0.92, 0.92, 0.0>]
    }
    scale 0.75
  }
  finish { ambient 1 }
  scale <5, 1, 6>
  translate <-14, 0.003, 11>
}
/*// right front mountain
disc { <0, 0, 0>, y, 1
  pigment {
    spherical
    turbulence 0.2
    colour_map {
      [0.00 colour rgbt <0.65, 0.65, 0.65, 1.0>]
      //[0.50 colour rgbt <0.65, 0.65, 0.65, 0.5>]
      [1.00 colour rgbt <0.65, 0.65, 0.65, 0.0>]
    }
    scale 0.75
  }
  finish { ambient 1 }
  scale <4, 1, 3>
  translate <-7, 0.004, 8>
}*/
// right mountain
disc { <0, 0, 0>, y, 1
  pigment {
    spherical
    turbulence 0.2
    colour_map {
      [0.00 colour rgbt <0.84, 0.84, 0.84, 1.0>]
      [0.50 colour rgbt <0.84, 0.84, 0.84, 0.7>]
      [1.00 colour rgbt <0.84, 0.84, 0.84, 0.0>]
    }
    scale 0.75
  }
  finish { ambient 1 }
  scale <5, 1, 6>
  translate <-6, 0.005, 10>
}

// shore
disc { <0, 0, 0>, y, 1
  pigment {
    spherical
    turbulence 0.1
    colour_map {
      [0.00 colour rgbt <0.20, 0.20, 0.20, 1.00>]
      [0.25 colour rgbt <0.20, 0.20, 0.20, 0.50>]
      [0.50 colour rgbt <0.00, 0.00, 0.00, 0.00>]
      [1.00 colour rgbt <0.00, 0.00, 0.00, 0.00>]
    }
    scale 0.75
  }
  finish { ambient 1 }
  scale <18, 1, 20>
  translate <12, 0.006, 10>
}
